
##LibName#DeskriptiveStat
##LibDescription#Funktionen zur Bestimmung und Darstellung von:
##LibDescription# - Werte der empirischer Verteilungsfunktion  
##LibDescription# - Prozentpunkte 
##LibDescription# - arithmetisches Mittel
##LibDescription# - Varianz 
##LibDescription# - Standardabweichung 
##LibDescription# - Kovarianz
##LibDescription#   Univariate Mazahlen und Funktionen aus Originaldaten und Hufigkeitstabellen (nicht-klassiert und klassiert, GUI-Objekt oder mit table erzeugt) 
##LibCopyright# Rainer Schlittgen  

Verteil<-function(x,x0){ 
##FunctionName#Verteil
##FunctionDescription# Berechnung der empirischen Verteilungsfunktion fr einen univariaten Datensatz x  an den in x0 angegebenen Stellen
##FunctionDescription# Fr x sind erlaubt: Variable, Vektor, Hufigkeitstabelle
##FunctionDescription# (unklassiert und klassiert, als GUI-Objekt oder  mit table erzeugt)                                                    
##FunctionSyntax#Verteil(x,x0)
##FunctionDescription# 
  a <- rep(NA,length(x0))
  if (is.vector(x))
    { 
     n <- length(x) 
     for (i in 1:length(x0))
        { 
          b <-  x0[i] 
          a[i] <- length(x[x<=b])/n           
        }
    }
  if (is.table(x))
    {
     tmp<-as.data.frame.table(x)   
     tm1  <- unlist(as.vector(tmp[,1]))
     tm1<-chartr("[","]",tm1) 
     tm1<-chartr("(","]",tm1) 
     tm1<-chartr("]","  ",tm1)
     tm1<-unlist(as.vector(strsplit(tm1, ",", extended=FALSE))) 
     xi<-as.numeric(tm1) 
     tmp<- as.matrix(tmp)     
     ni<-as.numeric(tmp[,2])
     if (length(xi)==length(ni))
       {
         hi<-cumsum(ni)/sum(ni) 
         for (i in 1:length(x0)) { a[i]<-hi[length(xi[xi<=x0[i]])]  }   
       }
     if (length(xi)==(2*length(ni)))
       {
         xi <- xi[c(1,seq(2,length(xi),2))]  
         hi <- ni/sum(ni)
         hi <- append(0,hi)
         fi <- cumsum(hi)
         for (i in 1:length(x0))
            { 
              b<- x0[i]
              j <- length(xi[xi<=b])
              a[i] <- fi[j]+((x0[i]-xi[j])/(xi[j+1]-xi[j]))*hi[j+1]
             }
        } 
    } 
  if (mode(x)=="list")
    {
     Xi<-as.data.frame(x)
     tmp <- as.numeric(unlist(as.vector(strsplit(Xi[,1], " - ", extended=FALSE))))
     if (length(tmp)==length(Xi[,2]))
       {
         xi <- Xi[,1]
         ni <- Xi[,2] 
         hi<-cumsum(ni)/sum(ni) 
         for (i in 1:length(x0)) { a[i]<-hi[length(xi[xi<=x0[i]])]  }   
       }
     if (length(tmp)==(2*length(Xi[,2])))
       {
         xi <- tmp[c(1,seq(2,length(tmp),2))] 
         hi <- Xi[,2]/sum(Xi[,2]) 
         hi<-append(0,hi)
         fi<- cumsum(hi)
         for (i in 1:length(x0))
            { 
              b<- x0[i]
              j <- length(xi[xi<=b])
              a[i] <- fi[j]+((x0[i]-xi[j])/(xi[j+1]-xi[j]))*hi[j+1]
             }
        }       
      }     
  a }
      

Quantil<-function(x,p){
##FunctionName#Quantil
##FunctionDescription#Berechnung der empirischen Verteilungsfunktion fr einen univariaten Datensatz x fr die vorgegebenen Anteile p 
##FunctionDescription# Fr x sind erlaubt: Variable, Vektor, Hufigkeitstabelle
##FunctionDescription# (unklassiert und klassiert, als GUI-Objekt oder  mit table erzeugt)                                                    
##FunctionSyntax#Quantil(x,p)
##FunctionDescription# 

  q <- rep(0,length(p))
  if (is.vector(x))
    { 
     xs <- sort(x) 
     q <- xs[ceiling(p*length(x))]  
    }
  if (is.table(x)) 
   {
     tmp<-as.data.frame.table(x)   
     tm1  <- unlist(as.vector(tmp[,1]))
     tm1<-chartr("[","]",tm1) 
     tm1<-chartr("(","]",tm1) 
     tm1<-chartr("]","  ",tm1)
     tm1<-unlist(as.vector(strsplit(tm1, ",", extended=FALSE))) 
     xi<-as.numeric(tm1) 
     tmp<- as.matrix(tmp)     
     ni<-as.numeric(tmp[,2])
     h <- ni/sum(ni)
     f <-cumsum(h)
     if (length(xi)==length(ni))
       {
         for (i in 1:length(p)) 
           { 
             d<-f-p[i]
             q[i]<-xi[length(d[d<0])+1] 
           }
       }
     if (length(xi)==(2*length(ni)))
        {
          xi <- xi[c(1,seq(2,length(tmp),2))] 
          h <- ni/sum(ni)
          f <- append(0,cumsum(h))
          for (i in 1:length(p))
             { 
               d <- f-p[i]
               j <- length(d[d<0])
               q[i] <- xi[j]+((p[i]-f[j])/h[j])*(xi[j+1]-xi[j])
              }
        }
    } 
  if (mode(x)=="list")
    {
      Xi<-as.data.frame(x)
      tmp <- as.numeric(unlist(as.vector(strsplit(Xi[,1], " - ", extended=FALSE))))
      if (length(tmp)==length(Xi[,2]))
        {
          xi<-as.numeric(Xi[,1])
          ni<-as.numeric(Xi[,2])
          h <-cumsum(ni)/sum(ni)
          for (i in 1:length(p))
             {
               d<-h-p[i]
               q[i]<-xi[length(d[d<0])+1]
             }
        }
      if (length(tmp)==(2*length(Xi[,2])))
        {
          xi <- tmp[c(1,seq(2,length(tmp),2))]
          ni <- as.numeric(Xi[,2])
          h <- ni/sum(ni)
          f <- append(0,cumsum(h))
          for (i in 1:length(p))
             { 
               b <- p[i]
               j <- length(f[b>=f])
               q[i] <- xi[j]+((p[i]-f[j])/h[j])*(xi[j+1]-xi[j])
              }
        }
    }
 q }
      
 
Mittel<-function(x){
##FunctionName#Mittel 
##FunctionDescription#Berechnung des arithmetischen Mittels  
##FunctionDescription# Fr x sind erlaubt: Variable, Vektor, Hufigkeitstabelle
##FunctionDescription# (unklassiert und klassiert, als GUI-Objekt oder  mit table erzeugt)   
##FunctionSyntax#Mittel( x )
##FunctionDescription# 
  
  m <- NA
  if (is.vector(x)) {m<-mean(x)}
  if (is.table(x))
    {
     tmp<-as.data.frame.table(x)   
     tm1  <- unlist(as.vector(tmp[,1]))
     tm1<-chartr("[","]",tm1) 
     tm1<-chartr("(","]",tm1) 
     tm1<-chartr("]","  ",tm1)
     tm1<-unlist(as.vector(strsplit(tm1, ",", extended=FALSE))) 
     xi<-as.numeric(tm1) 
     tmp<- as.matrix(tmp)     
     ni<-as.numeric(tmp[,2])
     if (length(xi)==length(ni)) {  m<-sum(xi*ni)/sum(ni) }
     if (length(xi)==(2*length(ni)))
       { 
         xi <- 0.5*(xi[seq(1,length(xi),2)]+xi[seq(2,length(xi),2)])
         m<-sum(xi*ni)/sum(ni)
        } 
    } 
  if (mode(x)=="list")
    {
     Xi<-as.data.frame(x)
     tmp <- as.numeric(unlist(as.vector(strsplit(Xi[,1], " - ", extended=FALSE))))
     if (length(tmp)==length(Xi[,2]))
       {
         m<-sum(Xi[,1]*Xi[,2])/sum(Xi[,2])
       }
     if (length(tmp)==(2*length(Xi[,2])))
       {
         xi <- tmp 
         xi <- 0.5*(xi[seq(1,length(xi),2)]+xi[seq(2,length(xi),2)])          
         m <- sum(xi*Xi[,2])/sum(Xi[,2])
        }       
      }     
  m }
 
 


Varianz<-function(x){
##FunctionName#Varianz 
##FunctionDescription#Berechnung der Varianz  
##FunctionDescription# Fr x sind erlaubt: Variable, Vektor, Hufigkeitstabelle
##FunctionDescription# (unklassiert und klassiert, als GUI-Objekt oder  mit table erzeugt) 
##FunctionDescription# Hier wird der Faktor 1/n und nicht 1/(n-1) verwendet ! 
##FunctionSyntax#Varianz( x )
##FunctionDescription# 
 
 v<-NA 
  if (is.vector(x)) {v<-var(x)*(length(x)-1)/length(x)}
  if (is.table(x))
    {
     tmp<-as.data.frame.table(x)   
     tm1  <- unlist(as.vector(tmp[,1]))
     tm1<-chartr("[","]",tm1) 
     tm1<-chartr("(","]",tm1) 
     tm1<-chartr("]","  ",tm1)
     tm1<-unlist(as.vector(strsplit(tm1, ",", extended=FALSE))) 
     xi<-as.numeric(tm1) 
     tmp<- as.matrix(tmp)     
     ni<-as.numeric(tmp[,2])
     if (length(xi)==length(ni)) 
       {  
         m<-sum(xi*ni)/sum(ni) 
         v<-sum(((xi-m)^2)*ni)/sum(ni)
       }
     if (length(xi)==(2*length(ni)))
       { 
         xi <- 0.5*(xi[seq(1,length(xi),2)]+xi[seq(2,length(xi),2)])
         m<-sum(xi*ni)/sum(ni)
         v<-sum(((xi-m)^2)*ni)/sum(ni)
        } 
    } 
  if (mode(x)=="list")
    {
     Xi<-as.data.frame(x)
     tmp <- as.numeric(unlist(as.vector(strsplit(Xi[,1], " - ", extended=FALSE))))
     ni <- Xi[,2]
     if (length(tmp)==length(ni))
       {
         xi <- Xi[,1]
         m<-sum(xi*ni)/sum(ni)
         v<-sum(((xi-m)^2)*ni)/sum(ni)
       }
     if (length(tmp)==(2*length(ni)))
       {
         xi <- tmp 
         xi <- 0.5*(xi[seq(1,length(xi),2)]+xi[seq(2,length(xi),2)])          
         m <- sum(xi*ni)/sum(ni)
         v<-sum(((xi-m)^2)*ni)/sum(ni)
        }       
      }     
  v }


Standabw<-function(x){
##FunctionName#Standabw 
##FunctionDescription#Berechnung der Standardabweichung  
##FunctionDescription# Fr x sind erlaubt: Variable, Vektor, Hufigkeitstabelle
##FunctionDescription# (unklassiert und klassiert, als GUI-Objekt oder  mit table erzeugt) 
##FunctionDescription# Hier wird der Faktor 1/n und nicht 1/(n-1) verwendet ! 
##FunctionSyntax#Standabw( x )
##FunctionDescription# 
 
 v<-NA 
  if (is.vector(x)) {v<-sqrt(var(x)*(length(x)-1)/length(x))}
  if (is.table(x))
    {
     tmp<-as.data.frame.table(x)   
     tm1  <- unlist(as.vector(tmp[,1]))
     tm1<-chartr("[","]",tm1) 
     tm1<-chartr("(","]",tm1) 
     tm1<-chartr("]","  ",tm1)
     tm1<-unlist(as.vector(strsplit(tm1, ",", extended=FALSE))) 
     xi<-as.numeric(tm1) 
     tmp<- as.matrix(tmp)     
     ni<-as.numeric(tmp[,2])
     if (length(xi)==length(ni)) 
       {  
         m<-sum(xi*ni)/sum(ni) 
         v<-sqrt(sum(((xi-m)^2)*ni)/sum(ni))
       }
     if (length(xi)==(2*length(ni)))
       { 
         xi <- 0.5*(xi[seq(1,length(xi),2)]+xi[seq(2,length(xi),2)])
         m<-sum(xi*ni)/sum(ni)
         v<-sqrt(sum(((xi-m)^2)*ni)/sum(ni))
        } 
    } 
  if (mode(x)=="list")
    {
     Xi<-as.data.frame(x)
     tmp <- as.numeric(unlist(as.vector(strsplit(Xi[,1], " - ", extended=FALSE))))
     ni <- Xi[,2]
     if (length(tmp)==length(ni))
       {
         xi <- Xi[,1]
         m<-sum(xi*ni)/sum(ni)
         v<-sqrt(sum(((xi-m)^2)*ni)/sum(ni))
       }
     if (length(tmp)==(2*length(ni)))
       {
         xi <- tmp 
         xi <- 0.5*(xi[seq(1,length(xi),2)]+xi[seq(2,length(xi),2)])          
         m <- sum(xi*ni)/sum(ni)
         v<-sqrt(sum(((xi-m)^2)*ni)/sum(ni))
        }       
      }     
  v }


Kovarianz<-function(x,y){
##FunctionName#Kovarianz
##FunctionDescription#Berechnung der Kovarianz  
##FunctionDescription# Fr x sind erlaubt: Variablen, Vektoren, Spalten einer Matrix
##FunctionDescription# Hier wird der Faktor 1/n und nicht 1/(n-1) verwendet ! 
##FunctionSyntax#Kovarianz( x,y )
##FunctionDescription# 
 
  v<-NA 
  v<- cov(x,y)*(length(x)-1)/length(x)
 }

Median<-function(x){
##FunctionName#Median
##FunctionDescription#Berechnung des Medians fr einen univariaten Datensatz x fr die vorgegebenen Anteile p 
##FunctionDescription# Fr x sind erlaubt: Variable, Vektor, Hufigkeitstabelle
##FunctionDescription# (unklassiert und klassiert, als GUI-Objekt oder  mit table erzeugt)                                                    
##FunctionSyntax#Median(x)
##FunctionDescription# 

  q <- NA
  if (is.vector(x)) { q<-median(x) }
  if (is.table(x))  
   {
     tmp<-as.data.frame.table(x)   
     tm1  <- unlist(as.vector(tmp[,1]))
     tm1<-chartr("[","]",tm1) 
     tm1<-chartr("(","]",tm1) 
     tm1<-chartr("]","  ",tm1)
     tm1<-unlist(as.vector(strsplit(tm1, ",", extended=FALSE))) 
     xi<-as.numeric(tm1) 
     tmp<- as.matrix(tmp)     
     ni<-as.numeric(tmp[,2])
     n<-sum(ni)
     h <- ni/sum(ni)
     f <-cumsum(h)
     if (length(xi)==length(ni))
       { q <- 0.5*(Quantil(x,ceiling(n/2))+Quantil(x,ceiling((n+1)/2))) }
     if (length(xi)==(2*length(ni)))
        { q <- Quantil(x,0.5) }
    } 
  if (mode(x)=="list")
    {
      Xi<-as.data.frame(x)
      tmp <- as.numeric(unlist(as.vector(strsplit(Xi[,1], " - ", extended=FALSE))))
      ni <- as.numeric(Xi[,2])
      n<-sum(ni)
      if (length(tmp)==length(ni))      
       { q <- 0.5*(Quantil(x,ceiling(n/2))+Quantil(x,ceiling((n+1)/2))) }
      if (length(tmp)==(2*length(ni)))
        { q <- Quantil(x,0.5) }
    }
 q }

 

Plotverteil<-function(x){ 
##FunctionName#Plotverteil
##FunctionDescription# Plot der empirischen Verteilungsfunktion fr einen univariaten Datensatz x  
##FunctionDescription# Fr x sind erlaubt: Variable, Vektor, Hufigkeitstabelle
##FunctionDescription# (unklassiert und klassiert, als GUI-Objekt oder  mit table erzeugt)                                                    
##FunctionSyntax#Plotverteil(x)
##FunctionDescription# 
   
  if (is.vector(x))
    { 
       n<-length(x)
       x<-sort(x)
       e<-(x[n]-x[1])/50
       x<-c((x[1]-e),x,(x[n]+e))
       plot(x,c(0,c(1:n)/n,1),type="s",ylab="F^(x)",xlab="x",lwd=2)
    }
  if (is.table(x))
    {
     tmp<-as.data.frame.table(x)   
     tm1  <- unlist(as.vector(tmp[,1]))
     tm1<-chartr("[","]",tm1) 
     tm1<-chartr("(","]",tm1) 
     tm1<-chartr("]","  ",tm1)
     tm1<-unlist(as.vector(strsplit(tm1, ",", extended=FALSE))) 
     xi<-as.numeric(tm1) 
     tmp<- as.matrix(tmp)     
     ni<-as.numeric(tmp[,2])
     if (length(xi)==length(ni))
       {
         hi<-cumsum(ni)/sum(ni)
         e<-(max(xi)-min(xi))/50
         xi<-c((xi[1]-e),xi,(xi[length(xi)]+e)) 
         plot(xi,c(0,hi,1),type="s",ylab="F^(x)",xlab="x",lwd=2)  
       }
     if (length(xi)==(2*length(ni)))
       {
         xi <- xi[c(1,seq(2,length(xi),2))]  
         e<-(max(xi)-min(xi))/50
         xi<-c((xi[1]-e),xi,(xi[length(xi)]+e)) 
         hi <- ni/sum(ni)
         hi <- append(0,hi)
         fi <- c(0,cumsum(hi),1)
         plot(xi,fi,type="l",ylab="F^(x)",xlab="x",lwd=2)
        } 
    } 
  if (mode(x)=="list")
    {
     Xi<-as.data.frame(x)
     tmp <- as.numeric(unlist(as.vector(strsplit(Xi[,1], " - ", extended=FALSE))))
     if (length(tmp)==length(Xi[,2]))
       {
         xi <- Xi[,1]
         ni <- Xi[,2] 
         hi<-cumsum(ni)/sum(ni) 
         e<-(max(xi)-min(xi))/50
         xi<-c((xi[1]-e),xi,(xi[length(xi)]+e)) 
         plot(xi,c(0,hi,1),type="s",ylab="F^(x)",xlab="x",lwd=2) 
       }
     if (length(tmp)==(2*length(Xi[,2])))
       {
         xi <- tmp[c(1,seq(2,length(tmp),2))]  
         e<-(max(xi)-min(xi))/50
         xi<-c((xi[1]-e),xi,(xi[length(xi)]+e))  
         hi <- Xi[,2]/sum(Xi[,2]) 
         hi<-append(0,hi)
         fi <- c(0,cumsum(hi),1)
         plot(xi,fi,type="l",ylab="F^(x)",xlab="x",lwd=2)
        }       
      }     
    }
 